
var TraceContextManager = require('../trace/trace-context-manager'),
    SqlStepX            = require('../step/sql-stepx'),
    DataTextAgent       = require('../data/datatext-agent'),
    HashUtil            = require('../util/hashutil');

var MongooseObserver = function (agent) {
    this.agent = agent;
    this.packages = ['mongoose'];
};

MongooseObserver.prototype.inject = function (mod, modName) {

    var self = this;

    var hookCommand = [
        'find',
        'remove',
        'findById',
        'findOne',
        'count',
        'distinct',
        'findOneAndUpdate',
        'findByIdAndUpdate',
        'findOneAndRemove',
        'findByIdAndRemove',
        'create',
        'insertMany',
        'update',
        'updateMany',
        'updateOne',
        'mapReduce',
        'geoNear',
        'geoSearch',
        'populate'
    ];

    self.agent.aop.before(mod.Model, 'aggregate', function (obj, args) {
        var ctx = TraceContextManager.getCurrentContext();
        if(ctx == null) {return;}

        var param = "";
        if(Array.isArray(args[0])) {
            args[0].forEach(function (val, i) {
                if(i > 0 && param.length > 0) {
                    param += ",";
                }
                if(val.hasOwnProperty('$match')) {
                    var keys = Object.keys(val['$match']);
                    param += keys;
                }

                if(val.hasOwnProperty('$group')) {
                    var keys = Object.keys(val['$group']);
                    param += keys;
                }
            })
        }

        var sql_step = new SqlStepX();
        var sql = 'mongodb aggregate (' + param + ')';
        sql_step.hash = HashUtil.hashFromString(sql);
        sql_step.start_time = ctx.getElapsedTime();
        sql_step.dbc = 0;
        
        DataTextAgent.SQL.add(sql_step.hash, sql);
        ctx.profile.push(sql_step);

        self.agent.aop.functionHook(args, -1, function () {
            TraceContextManager.resume(ctx);
            sql_step.elapsed = ctx.getElapsedTime() - sql_step.start_time;
        });
    });

    self.agent.aop.before(mod.Model, hookCommand, function (obj, args) {
        var ctx_id = TraceContextManager.getCurrentId();
        self.agent.aop.functionHook(args, -1, function () {
            TraceContextManager.resume(ctx_id);
        });
    });

};

exports.MongooseObserver = MongooseObserver;