/**
 * Copyright 2016 the WHATAP project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */

var TraceContextManager = require('../trace/trace-context-manager'),
    SocketStep = require('../step/socket-step'),
    conf = require('../conf/configure'),
    IPUtil = require('../util/iputil'),
    Logger = require('../logger');


var NetObserver = function (agent) {
    this.agent = agent;
    this.packages = ['net', 'dgram'];
};

NetObserver.prototype.inject = function (mod, moduleName) {
    if (mod.__whatap_observe__) {
        return;
    }
    mod.__whatap_observe__ = true;
    Logger.initPrint("NetObserver");
    if (conf.trace_background_socket_enabled === false) {
        return;
    }

    var self = this;
    var aop = self.agent.aop;

    aop.both(mod.Socket.prototype, ['connect'],
        function (obj, args, lctx) {
            var ctx = lctx.context;
            if (ctx == null) {
                return;
            }
            var host, port;
            if (args.length > 1) {
                host = args[1];
                port = args[0];
            }
            if (port == undefined) {
                lctx.step = null;
                return;
            }
            ctx.socket_connecting = true;

            ctx.footprint('Socket Connecting: ' + host + ':' + port);

            var step = new SocketStep();
            step.start_time = ctx.getElapsedTime();
            step.ipaddr = Buffer.from(IPUtil.stringToBytes(host));
            step.port = port;
            ctx.profile.push(step);
            lctx.step = step;
        },
        function (obj, args, ret, lctx) {
            var ctx = lctx.context;
            var step = lctx.step;
            if (ctx == null || step == null) {
                return;
            }
            ctx.socket_connecting = false;
            step.elapsed = ctx.getElapsedTime() - step.start_time;
            ctx.footprint('Socket Connecting Done');
        }
    );

    aop.both(mod, ['createSocket'],
        function (obj, args, lctx) {
            var ctx = lctx.context;
            if (ctx == null) {
                return;
            }
        },
        function (obj, args, ret, lctx) {
            var ctx = lctx.context;
            if (ctx == null) {
                return;
            }
            aop.after(ret, 'send', function (_obj, _args, _ret, _lctx) {
                var host, port;
                if (_args.length > 2) {
                    host = _args[2];
                    port = _args[1];
                }
                if (port == undefined) {
                    lctx.step = null;
                    return;
                }
                ctx.socket_connecting = true;

                ctx.footprint('Socket Connecting: ' + host + ':' + port);

                var step = new SocketStep();
                step.start_time = ctx.getElapsedTime();
                step.ipaddr = Buffer.from(IPUtil.stringToBytes(host));
                step.port = port;

                ctx.socket_connecting = false;
                step.elapsed = ctx.getElapsedTime() - step.start_time;
                ctx.profile.push(step);

                ctx.footprint('Socket Connecting Done');
            })
        }
    );
};

exports.NetObserver = NetObserver;